<?php

namespace Litecms\Crypto\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Crypto\Http\Requests\CryptoRequest;
use Litecms\Crypto\Interfaces\CryptoRepositoryInterface;
use Litecms\Crypto\Models\Crypto;

/**
 * Resource controller class for crypto.
 */
class CryptoResourceController extends BaseController
{

    /**
     * Initialize crypto resource controller.
     *
     * @param type CryptoRepositoryInterface $crypto
     *
     * @return null
     */
    public function __construct(CryptoRepositoryInterface $crypto)
    {
        parent::__construct();
        $this->repository = $crypto;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Crypto\Repositories\Criteria\CryptoResourceCriteria::class);
    }

    /**
     * Display a list of crypto.
     *
     * @return Response
     */
    public function index(CryptoRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Crypto\Repositories\Presenter\CryptoPresenter::class)
                ->$function();
        }

        $cryptos = $this->repository->paginate();

        return $this->response->title(trans('crypto::crypto.names'))
            ->view('crypto::crypto.index', true)
            ->data(compact('cryptos'))
            ->output();
    }

    /**
     * Display crypto.
     *
     * @param Request $request
     * @param Model   $crypto
     *
     * @return Response
     */
    public function show(CryptoRequest $request, Crypto $crypto)
    {

        if ($crypto->exists) {
            $view = 'crypto::crypto.show';
        } else {
            $view = 'crypto::crypto.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('crypto::crypto.name'))
            ->data(compact('crypto'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new crypto.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CryptoRequest $request)
    {

        $crypto = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('crypto::crypto.name')) 
            ->view('crypto::crypto.create', true) 
            ->data(compact('crypto'))
            ->output();
    }

    /**
     * Create new crypto.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CryptoRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $crypto                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('crypto::crypto.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('crypto/crypto/' . $crypto->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/crypto/crypto'))
                ->redirect();
        }

    }

    /**
     * Show crypto for editing.
     *
     * @param Request $request
     * @param Model   $crypto
     *
     * @return Response
     */
    public function edit(CryptoRequest $request, Crypto $crypto)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('crypto::crypto.name'))
            ->view('crypto::crypto.edit', true)
            ->data(compact('crypto'))
            ->output();
    }

    /**
     * Update the crypto.
     *
     * @param Request $request
     * @param Model   $crypto
     *
     * @return Response
     */
    public function update(CryptoRequest $request, Crypto $crypto)
    {
        try {
            $attributes = $request->all();

            $crypto->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('crypto::crypto.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('crypto/crypto/' . $crypto->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('crypto/crypto/' . $crypto->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the crypto.
     *
     * @param Model   $crypto
     *
     * @return Response
     */
    public function destroy(CryptoRequest $request, Crypto $crypto)
    {
        try {

            $crypto->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('crypto::crypto.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('crypto/crypto/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('crypto/crypto/' . $crypto->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple crypto.
     *
     * @param Model   $crypto
     *
     * @return Response
     */
    public function delete(CryptoRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('crypto::crypto.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('crypto/crypto'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/crypto/crypto'))
                ->redirect();
        }

    }

    /**
     * Restore deleted cryptos.
     *
     * @param Model   $crypto
     *
     * @return Response
     */
    public function restore(CryptoRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('crypto::crypto.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/crypto/crypto'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/crypto/crypto/'))
                ->redirect();
        }

    }

}
